package com.soda_is.empclimate;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.StringWriter;
import java.net.URL;
import java.net.URLConnection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.jws.WebResult;
import javax.jws.WebService;
import javax.jws.soap.SOAPBinding;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.Duration;

/* [Tutorial comment] Import of types defined from the SolarResourceKnowledge.xsd */
import com.soda_is.solarresourceknowledge.*;

/**
 * This class was generated by the JAX-WS RI. JAX-WS RI 2.1.1-b03- Generated
 * source version: 2.0
 * 
 */
/*
 * [Tutorial comment] The EMPClimatePortType type has been changed from
 * interface type to class type
 */
@WebService(name = "EMPClimatePortType", targetNamespace = "http://www.soda-is.com/EMPClimate")
@SOAPBinding(style = SOAPBinding.Style.RPC)
public class EMPClimatePortType {

	/**
	 * 
	 * @param getAllValuesRequest
	 * @return returns com.soda_is.empclimate.GetAllValuesResponseType
	 */
	@WebMethod(operationName = "GetAllValues", action = "http://www.soda-is.com/EMPClimat/GetAllValues")
	@WebResult(name = "GetAllValuesResponse", partName = "GetAllValuesResponse")
	public GetAllValuesResponseType getAllValues(
			@WebParam(name = "GetAllValuesRequest", partName = "GetAllValuesRequest") GetAllValuesRequestType getAllValuesRequest) {
		/*
		 * [Tutorial comment] DataTypeFactory belongs to the javax.xml.datatype
		 * It creates new javax.xml.datatype Objects to/from xml
		 */
		DatatypeFactory df = null;

		/*
		 * [Tutorial comment] Get the lat/lon from the geopoint in the
		 * getAllValuesRequest object
		 */
		String s_lat = String.valueOf(getAllValuesRequest.getGeopoint()
				.getLatitude());
		String s_lon = String.valueOf(getAllValuesRequest.getGeopoint()
				.getLongitude());

		InputStream txt_result = null;

		/*
		 * [Tutorial comment] Creation of a hashmap for the parameters list for
		 * the http_request of http://www.helioclim.net/com/ncep_climat_gsc.php
		 * (GET method) Hashmap structure : parameter_name | parameter
		 */
		HashMap<String, String> http_request_data = new HashMap<String, String>();
		http_request_data.put("latlon", s_lat + "," + s_lon);

		/*
		 * [Tutorial comment] Creation of a hashmap for the output of
		 * ncep_climat_gsc.php
		 */
		HashMap<String, String[]> data = null;

		try {

			df = DatatypeFactory.newInstance();

			/*
			 * [Tutorial comment] Call of ncep_climate.php through a private
			 * method getResults defined in this java file - creation of the url
			 * call (get method) : e.g.
			 * http://www.helioclim.net/com/ncep_climate.php?latlon=45,4 -
			 * redirection of the output of ncep_climate.php in the
			 * InputStream txt_result
			 */
			txt_result = getResults("get",
					"http://www.helioclim.net/com/ncep_climate.php",
					http_request_data);

			/*
			 * [Tutorial comment] Parsing of the InputStream txt_result to
			 * "populate" the hasmap data Hashmap structure : type_name |
			 * related_data type_name = "#GHI" or "#TempMin", etc. related_data
			 * "lat lon altitude measure_jan measure_feb ... measure_dec
			 * measure_annual
			 */
			data = getData(txt_result);

			txt_result.close();

		} catch (Exception e) {
			e.printStackTrace();
		}

		/*
		 * [Tutorial comment] The ascii stream System.out is redirected in the
		 * Jboss console
		 */
		System.out.println(data.toString());

		/*
		 * [Tutorial comment] Definition of the IPR Object as defined in the
		 * SolarResourceKnowledge.xsd
		 */
		IPRType IPR = new IPRType();

		IPR.setCopyrightText("Unknow");
		IPR.setProviderName("MINES ParisTech");
		IPR.setProviderURL("http://www.soda-is.com/");
		IPR.setProviderLogoURL("http://www.ensmp.fr/Fr/Services/" +
				"LibresServices/visu-logo/images/logo-MinesParisTech.png");

		/* [Tutorial comment] Creation & definition of location (geopoint) */
		GeopointType location = getAllValuesRequest.getGeopoint();

		/*
		 * [Tutorial comment] Creation & definition of the
		 * SequenceOfGlobalIrradiance Object for GlobalIrradiance as defined in
		 * the SolarResourceKnowledge.xsd
		 */
		
		Duration one_month = df.newDurationYearMonth(true, 0, 1);
		
		SequenceOfObservationType seq_ghi = new SequenceOfObservationType();
		seq_ghi.setAbsenceOfValue(-999.0f);
		seq_ghi.setOffsetInstant(1.0f);
		seq_ghi.setDuration(one_month);
		seq_ghi.setTimeSeriesTitle("EMPClimate Monthly GHI Time Serie");
		seq_ghi.setObservationType("Unknow");
		seq_ghi.setIPR(IPR);
		seq_ghi.setLocation(location);

		/*
		 * [Tutorial comment] Creation of an element of a
		 * SequenceOfObservation as list of
		 * SingleObservationType
		 */
		List<SingleObservationType> global_seq = seq_ghi.getSequence();

		/*
		 * [Tutorial comment] Use of the hasmap data with the entry-key "#GHI"
		 * to get the related data from
		 * http://www.helioclim.net/com/ncep_climate.php for each months (3
		 * to n-1) row[0] corresponds to lat row[1] corresponds to lon row[2]
		 * corresponds to altitude row[3 to row.length-1] corresponds to GHI of
		 * jan, ..., dec
		 */
		String[] row = data.get("#GHI");
		for (int i = 3; i < row.length - 1; i++) {
			String cur_value = row[i];
			SingleObservationType obs = new SingleObservationType();
			float value = Float.valueOf(cur_value);
			obs.setDate(df.newXMLGregorianCalendar(2000, i - 2, 1, 0, 0, 0, 0, 0));
			obs.setRelability(-999.0f);
			obs.setUncertainty(-999.0f);
			obs.setValue(value);
			global_seq.add(obs);
		}

		/*
		 * [Tutorial comment] The same as previously for Min temperature
		 * sequence
		 */
		SequenceOfObservationType seq_tempmin = new SequenceOfObservationType();
		seq_tempmin.setAbsenceOfValue(-999.0f);
		seq_tempmin.setOffsetInstant(1.0f);
		seq_tempmin.setDuration(one_month);
		seq_tempmin.setTimeSeriesTitle("EMPClimate Monthly #TempMin Time Serie");
		seq_tempmin.setObservationType("Unknow");
		seq_tempmin.setIPR(IPR);
		seq_tempmin.setLocation(location);


		List<SingleObservationType> mintemp_seq = seq_tempmin.getSequence();

		row = data.get("#TempMin");
		for (int i = 3; i < row.length - 2; i++) {
			String cur_value = row[i];
			SingleObservationType obs = new SingleObservationType();
			float value = Float.valueOf(cur_value);
			obs.setDate(df.newXMLGregorianCalendar(
					2000, i - 2, 1, 0, 0, 0, 0, 0));
			obs.setRelability(-999.0f);
			obs.setUncertainty(-999.0f);
			obs.setValue(value);
			mintemp_seq.add(obs);
		}

		/*
		 * [Tutorial comment] The same as previously for Max temperature
		 * sequence
		 */
		SequenceOfObservationType seq_tempmax = new SequenceOfObservationType();
		seq_tempmax.setAbsenceOfValue(-999.0f);
		seq_tempmax.setOffsetInstant(1.0f);
		seq_tempmax.setDuration(one_month);
		seq_tempmax.setTimeSeriesTitle("EMPClimate Monthly #TempMax Time Serie");
		seq_tempmax.setObservationType("Unknow");
		seq_tempmax.setIPR(IPR);
		seq_tempmax.setLocation(location);


		List<SingleObservationType> maxtemp_seq = seq_tempmax.getSequence();

		row = data.get("#TempMax");
		for (int i = 3; i < row.length - 2; i++) {
			String cur_value = row[i];
			SingleObservationType obs = new SingleObservationType();
			float value = Float.valueOf(cur_value);
			obs.setDate(df.newXMLGregorianCalendar(
					2000, i - 2, 1, 0, 0, 0, 0, 0));
			obs.setRelability(-999.0f);
			obs.setUncertainty(-999.0f);
			obs.setValue(value);
			maxtemp_seq.add(obs);
		}

		/*
		 * [Tutorial comment] The same as previously for Mean temperature
		 * sequence
		 */
		
		System.out.println("*************************************");
		SequenceOfObservationType seq_tempmean = new SequenceOfObservationType();
		seq_tempmean.setAbsenceOfValue(-999.0f);
		seq_tempmean.setOffsetInstant(1.0f);
		seq_tempmean.setDuration(one_month);
		seq_tempmean.setTimeSeriesTitle("EMPClimate Monthly #TempMean Time Serie");
		seq_tempmean.setObservationType("Unknow");
		seq_tempmean.setIPR(IPR);
		seq_tempmean.setLocation(location);


		List<SingleObservationType> meantemp_seq = seq_tempmean.getSequence();

		row = data.get("#TempMean");
		for (int i = 3; i < row.length - 2; i++) {
			String cur_value = row[i];
			SingleObservationType obs = new SingleObservationType();
			float value = Float.valueOf(cur_value);
			obs.setDate(df.newXMLGregorianCalendar(
					2000, i - 2, 1, 0, 0, 0, 0, 0));
			obs.setRelability(-999.0f);
			obs.setUncertainty(-999.0f);
			obs.setValue(value);
			meantemp_seq.add(obs);
		}

		/*
		 * [Tutorial comment] The same as previously for Min humidity
		 * sequence
		 */
		SequenceOfObservationType seq_hummin = new SequenceOfObservationType();
		seq_hummin.setAbsenceOfValue(-999.0f);
		seq_hummin.setOffsetInstant(1.0f);
		seq_hummin.setDuration(one_month);
		seq_hummin.setTimeSeriesTitle("EMPClimate Monthly #HumMin Time Serie");
		seq_hummin.setObservationType("Unknow");
		seq_hummin.setIPR(IPR);
		seq_hummin.setLocation(location);


		List<SingleObservationType> minhum_seq = seq_hummin.getSequence();

		row = data.get("#HumMin");
		for (int i = 3; i < row.length - 2; i++) {
			String cur_value = row[i];
			SingleObservationType obs = new SingleObservationType();
			float value = Float.valueOf(cur_value);
			obs.setDate(df.newXMLGregorianCalendar(
					2000, i - 2, 1, 0, 0, 0, 0, 0));
			obs.setRelability(-999.0f);
			obs.setUncertainty(-999.0f);
			obs.setValue(value);
			minhum_seq.add(obs);
		}

		/*
		 * [Tutorial comment] The same as previously for Max humidity
		 * sequence
		 */
		SequenceOfObservationType seq_hummax = new SequenceOfObservationType();
		seq_hummax.setAbsenceOfValue(-999.0f);
		seq_hummax.setOffsetInstant(1.0f);
		seq_hummax.setDuration(one_month);
		seq_hummax.setTimeSeriesTitle("EMPClimate Monthly #HumMax Time Serie");
		seq_hummax.setObservationType("Unknow");
		seq_hummax.setIPR(IPR);
		seq_hummax.setLocation(location);


		List<SingleObservationType> maxhum_seq = seq_hummax.getSequence();

		row = data.get("#HumMax");
		for (int i = 3; i < row.length - 2; i++) {
			String cur_value = row[i];
			SingleObservationType obs = new SingleObservationType();
			float value = Float.valueOf(cur_value);
			obs.setDate(df.newXMLGregorianCalendar(
					2000, i - 2, 1, 0, 0, 0, 0, 0));
			obs.setRelability(-999.0f);
			obs.setUncertainty(-999.0f);
			obs.setValue(value);
			maxhum_seq.add(obs);
		}

		/*
		 * [Tutorial comment] The same as previously for Mean humidity
		 * sequence
		 */
		SequenceOfObservationType seq_hummean = new SequenceOfObservationType();
		seq_hummean.setAbsenceOfValue(-999.0f);
		seq_hummean.setOffsetInstant(1.0f);
		seq_hummean.setDuration(one_month);
		seq_hummean.setTimeSeriesTitle("EMPClimate Monthly #HumMean Time Serie");
		seq_hummean.setObservationType("Unknow");
		seq_hummean.setIPR(IPR);
		seq_hummean.setLocation(location);


		List<SingleObservationType> meanhum_seq = seq_hummean.getSequence();

		row = data.get("#HumMean");
		for (int i = 3; i < row.length - 2; i++) {
			String cur_value = row[i];
			SingleObservationType obs = new SingleObservationType();
			float value = Float.valueOf(cur_value);
			obs.setDate(df.newXMLGregorianCalendar(
					2000, i - 2, 1, 0, 0, 0, 0, 0));
			obs.setRelability(-999.0f);
			obs.setUncertainty(-999.0f);
			obs.setValue(value);
			meanhum_seq.add(obs);
		}

		/*
		 * [Tutorial comment] Creation and definition of the output type as
		 * defined in the wsdl
		 */
		GetAllValuesResponseType result = new GetAllValuesResponseType();
		result.setGhi(seq_ghi);
		result.setTempMin(seq_tempmin);
		result.setTempMax(seq_tempmax);
		result.setTempMean(seq_tempmean);
		result.setHumMin(seq_hummin);
		result.setHumMax(seq_hummax);
		result.setHumMean(seq_hummean);

		return result;

	}

	/* [Tutorial comment] Private method for the url call of the php */
	
	private InputStream getResults(String s_method, String s_url,
			HashMap<String, String> parameters) throws Exception {
		try {

			String s_parameters = "";
			Iterator<String> it = parameters.keySet().iterator();
			while (it.hasNext()) {
				String current_key = it.next();
				s_parameters += current_key + "=" + parameters.get(current_key);
				if (it.hasNext()) {
					s_parameters += "&";
				}
			}

			System.out.println("[INFO] Call " + s_url + " : (" + s_method
					+ ") " + s_parameters);

			if (s_method.toLowerCase().equals("get")) {
				URL call = new URL(s_url + "?" + s_parameters);
				return call.openStream();
			} else if (s_method.toLowerCase().equals("post")) {

				URL url = new URL(s_url);
				URLConnection conn = url.openConnection();
				conn.setDoOutput(true);
				OutputStream request_stream = conn.getOutputStream();
				OutputStreamWriter wr = new OutputStreamWriter(request_stream);
				wr.write(s_parameters);
				wr.flush();
				wr.close();
				// Get reply.
				return conn.getInputStream();

			} else {
				// I don't know method
				throw new Exception("Unknow method : " + s_method);
			}
		} catch (FileNotFoundException e) {
			System.out.println("[ERROR] getResult : " +
					"something wrong with the URL or the temp file");
			e.printStackTrace();
		} catch (IOException ioe) {
			System.out.println("[ERROR] getResult : something wrong with I/O");
			ioe.printStackTrace();
		}
		return null;
	}

	/*
	 * [Tutorial comment] Privation (specific) function that parse the output of
	 * the php called by getResults
	 */
	private HashMap<String, String[]> getData(InputStream doc) throws Exception {

		String[] lines = readlines(doc);
		HashMap<String, String[]> result = new HashMap<String, String[]>();

		for (int i = 0; i < lines.length; i++) {
			String cur_line = lines[i];
			if (cur_line.startsWith("#")) {
				String[] data = cur_line.split("\\s+");
				String key = data[0];
				String[] value = new String[data.length - 1];
				for (int j = 1; j < data.length; j++)
					value[j - 1] = data[j];
				result.put(key, value);
			}
		}
		return result;
	}

	/*
	 * Simple & understandable date parsing.
	 */
	int[] parse_date(String date) {
		String[] t = date.split("-");
		int[] result = new int[3];
		result[0] = Integer.valueOf(t[0]).intValue();
		result[1] = Integer.valueOf(t[1]).intValue();
		result[2] = Integer.valueOf(t[2]).intValue();
		return result;
	}

	private String[] readlines(InputStream f) throws IOException {
		StringWriter s = new StringWriter();
		int c = -1;
		while ((c = f.read()) != -1)
			s.write(c);
		System.out.println(s);
		return s.toString().replaceAll("\r", "").split("\n");
	}

}
